/**
 * Copyright (C) 2009 Bull S.A.S.
 *
 * CAROL: Common Architecture for RMI ObjectWeb Layer
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: LMIContext.java 2206 2010-02-24 16:21:55Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.carol.jndi.spi;

import java.rmi.Remote;

import javax.naming.Context;
import javax.naming.Name;
import javax.naming.NamingException;
import javax.naming.Referenceable;

import org.ow2.carol.jndi.wrapping.UnicastJNDIReferenceWrapper;
import org.ow2.carol.rmi.exception.NamingExceptionHelper;

/**
 * Context used for intra-JVM calls.
 * 
 * @author Florent Benoit
 */
public class LMIContext extends AbsContext implements Context {

    /**
     * Constructor, load communication framework and instaciate initial contexts
     */
    public LMIContext(final Context ctx) throws NamingException {
        super(ctx);
    }

    /**
     * If this object is a reference wrapper return the reference If this object
     * is a resource wrapper return the resource
     * 
     * @param o
     *            the object to resolve
     * @param name
     *            name of the object to unwrap
     * @return the unwrapped object
     * @throws NamingException
     *             if the object cannot be unwraped
     */
    @Override
    protected Object unwrapObject(final Object o, final Name name) throws NamingException {
        return super.defaultUnwrapObject(o, name);
    }

    /**
     * Wrap an Object : If the object is a reference wrap it into a Reference
     * Wrapper Object here the good way is to contact the carol configuration to
     * get the portable remote object
     * 
     * @param o
     *            the object to encode
     * @param name
     *            of the object
     * @param replace
     *            if the object need to be replaced
     * @return a <code>Remote JNDIRemoteReference Object</code> if o is a
     *         resource o if else
     * @throws NamingException
     *             if object cannot be wrapped
     */
    @Override
    protected Object wrapObject(final Object o, final Name name, final boolean replace) throws NamingException {
        try {
            // Add wrapper for the two first cases. Then it will use
            // PortableRemoteObject instead of UnicastRemoteObject
            // and when fixing JRMP exported objects port, it use
            // JRMPProdelegate which is OK.
            if ((!(o instanceof Remote)) && (o instanceof Referenceable)) {
                return new UnicastJNDIReferenceWrapper(((Referenceable) o).getReference(), getObjectPort());
            } else {
                return o;
            }
        } catch (Exception e) {
            throw NamingExceptionHelper.create("Cannot wrap object '" + o + "' with name '" + name + "' : " + e.getMessage(), e);
        }
    }

}
