/**
 * Copyright (C) 2009 Bull S.A.S.
 *
 * CAROL: Common Architecture for RMI ObjectWeb Layer
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: LMIContextWrapperFactory.java 2206 2010-02-24 16:21:55Z benoitf $
 * --------------------------------------------------------------------------
 */

/**
 * Factory used for intra-JVM calls.
 * @author Florent Benoit
 */
package org.ow2.carol.jndi.spi;

import java.util.Hashtable;

import javax.naming.NamingException;
import javax.naming.spi.InitialContextFactory;

import org.ow2.carol.jndi.enc.java.CompNamingContext;

public class LMIContextWrapperFactory extends AbsInitialContextFactory<LMIContext> implements InitialContextFactory {

    /**
     * @return class of the wrapper (to be instantiated + pool).
     */
    @Override
    protected Class<LMIContext> getWrapperClass() {
        return LMIContext.class;
    }
    
    @Override
    protected String getReferencingFactory() {
        return LMIContextWrapperFactory.class.getName();
    }
    
    /**
     * Gets the values to give to the constructor of the given class (provided by getWrapperClass())
     * @param environment given environment
     * @return array of objects for the constructor
     * @throws NamingException if the new InitialContext fails
     */
    @Override
    protected Object[] getClassArgs(final Hashtable<?, ?> environment) throws NamingException {
        return new Object[] {new CompNamingContext("lmi", environment)};
    }


}
