/**
 * Copyright (C) 2008 - Bull S.A.S.
 *
 * CAROL: Common Architecture for RMI OW2 Layer
 *
 * This library is developed inside the OW2 Consortium,
 * http://www.ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JInterceptorConverter.java 2206 2010-02-24 16:21:55Z benoitf $
 * --------------------------------------------------------------------------
 */
package org.ow2.carol.rmi.jrmp.interceptor.impl.legacy;

import java.lang.reflect.Proxy;

import org.ow2.carol.rmi.interceptor.spi.JClientRequestInterceptor;
import org.ow2.carol.rmi.interceptor.spi.JServerRequestInterceptor;

/**
 * Contain methods to convert interceptors designed for Carol 2.x.
 * @author Loris Bouzonnet
 */
@SuppressWarnings("deprecation")
public class JInterceptorConverter {

    /**
     * Utility class: no constructor
     */
    private JInterceptorConverter() {
    }

    /**
     * Convert an array of server request interceptors designed for Carol 2.x to a list of them supported by Carol 3.x.
     * @param legacySis an array of server request interceptors designed for Carol 2.x
     * @return a list of server request interceptors supported by Carol 3.x
     */
    public static JServerRequestInterceptor[] convertJServerRequestInterceptor(
            final org.objectweb.carol.rmi.jrmp.interceptor.JServerRequestInterceptor[] legacySis) {
        JServerRequestInterceptor[] sis = new JServerRequestInterceptor[legacySis.length];
        int i = 0;
        for(org.objectweb.carol.rmi.jrmp.interceptor.JServerRequestInterceptor legacySi : legacySis) {
            JServerRequestInterceptorInvocationHandler jServerRequestInterceptorInvocationHandler =
                new JServerRequestInterceptorInvocationHandler(legacySi);
            sis[i] = (JServerRequestInterceptor) Proxy.newProxyInstance(
                        JInterceptorConverter.class.getClassLoader(),
                        new Class<?>[]{JServerRequestInterceptor.class},
                        jServerRequestInterceptorInvocationHandler);
            i++;
        }
        return sis;
    }

    /**
     * Convert an array of client request interceptors designed for Carol 2.x to a list of them supported by Carol 3.x.
     * @param legacyCis an array of client request interceptors designed for Carol 2.x
     * @return a list of client request interceptors supported by Carol 3.x
     */
    public static JClientRequestInterceptor[] convertJClientRequestInterceptor(
            final org.objectweb.carol.rmi.jrmp.interceptor.JClientRequestInterceptor[] legacyCis) {
        JClientRequestInterceptor[] cis = new JClientRequestInterceptor[legacyCis.length];
        int i = 0;
        for(org.objectweb.carol.rmi.jrmp.interceptor.JClientRequestInterceptor legacyCi : legacyCis) {
            JClientRequestInterceptorInvocationHandler jClientRequestInterceptorInvocationHandler =
                new JClientRequestInterceptorInvocationHandler(legacyCi);
            cis[i] = (JClientRequestInterceptor) Proxy.newProxyInstance(
                        JInterceptorConverter.class.getClassLoader(),
                        new Class<?>[]{JClientRequestInterceptor.class},
                        jClientRequestInterceptorInvocationHandler);
            i++;
        }
        return cis;
    }

    protected static String convertMethodName(final String methodName) {
        String result = "";
        int length = methodName.length();
        int index = 0;
        while(index != length) {
            char c = methodName.charAt(index);
            if(Character.isUpperCase(c)) {
                result = result.concat("_" + Character.toString(c).toLowerCase());
            } else {
                result = result.concat(Character.toString(c));
            }
            index++;
        }
        return result;
    
    }

}
