/**
 * Copyright (C) 2008 - Bull S.A.S.
 *
 * CAROL: Common Architecture for RMI OW2 Layer
 *
 * This library is developed inside the OW2 Consortium,
 * http://www.ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JServerRequestInterceptorInvocationHandler.java 2206 2010-02-24 16:21:55Z benoitf $
 * --------------------------------------------------------------------------
 */
package org.ow2.carol.rmi.jrmp.interceptor.impl.legacy;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import org.objectweb.carol.rmi.jrmp.interceptor.JServerRequestInfo;
import org.objectweb.carol.rmi.jrmp.interceptor.JServerRequestInterceptor;

/**
 * Handle invocations on a server request interceptor designed for Carol 2.x.
 * @author Loris Bouzonnet
 */
@SuppressWarnings("deprecation")
public class JServerRequestInterceptorInvocationHandler implements
        InvocationHandler {

    /**
     * A server request interceptor designed for Carol 2.x.
     */
    private final JServerRequestInterceptor legacySi;

    /**
     * Construct a new invocation handler for server request interceptors.
     * @param legacySi a server request interceptor designed for Carol 2.x
     */
    public JServerRequestInterceptorInvocationHandler(
            final JServerRequestInterceptor legacySi) {
        this.legacySi = legacySi;
    }

    public Object invoke(final Object proxy, final Method method, final Object[] args)
            throws Throwable {
        Object[] convertedArgs = args;
        Class<?>[] typeArgs = method.getParameterTypes();
        if(args != null
                && args.length == 1
                && (args[0] instanceof org.ow2.carol.rmi.interceptor.api.JServerRequestInfo)) {
            convertedArgs = new Object[] {convertJServerRequestInfo(
                    (org.ow2.carol.rmi.interceptor.api.JServerRequestInfo) args[0])};
            typeArgs = new Class<?>[]{JServerRequestInfo.class};
        }
        String methodName = JInterceptorConverter.convertMethodName(method.getName());
        Method legacyMethod = legacySi.getClass().getMethod(methodName, typeArgs);
        return legacyMethod.invoke(legacySi, convertedArgs);
    }

    /**
     * Convert server request info designed for Carol 3.x to be supported by server request interceptors designed for Carol 2.x.
     * @param jServerRequestInfo server request info designed for Carol 3.x
     * @return server request info designed for Carol 2.x
     */
    private JServerRequestInfo convertJServerRequestInfo(
            final org.ow2.carol.rmi.interceptor.api.JServerRequestInfo jServerRequestInfo) {
        JRequestInfoInvocationHandler jRequestInfoInvocationHandler =
            new JRequestInfoInvocationHandler(jServerRequestInfo);
        return (JServerRequestInfo) Proxy.newProxyInstance(
                getClass().getClassLoader(),
                new Class<?>[]{JServerRequestInfo.class},
                jRequestInfoInvocationHandler);
    }

}
